package com.foobar.controller;

import com.foobar.model.LoginHistory;
import com.foobar.model.User;
import com.foobar.service.LoginHistoryService;
import com.foobar.service.UserService;

import javax.enterprise.context.SessionScoped;
import javax.faces.application.FacesMessage;
import javax.faces.context.ExternalContext;
import javax.faces.context.FacesContext;
import javax.inject.Named;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.io.Serializable;
import java.util.Date;

@Named
@SessionScoped
public class SessionManager implements Serializable
{
    private static final long serialVersionUID = -2007778519739230625L;

    private User user;
    private boolean passwordChanged = false;

    public SessionManager() {}

    public boolean isLoggedIn() { return null != getRequest().getUserPrincipal(); }
    public boolean isLoggedOut() { return !isLoggedIn(); }

    public void login(String email, String password) throws Exception
    {
        getRequest().login(email, password);
        this.user = lookupEJB(UserService.class).findByEmail(email);
    }

    public void logoutSuccess()
    {
        createLoginHistoryRecord();
        logout();
    }

    public void logoutFailure()
    {
        logout();
        createLoginHistoryRecord();
    }

    private void logout()
    {
        try { getRequest().logout(); }
        catch(ServletException ex)
        {
            ex.printStackTrace();
            throw new IllegalStateException(ex);
        }
        HttpSession session = getRequest().getSession(false);
        if(null != session) { session.invalidate(); }
    }

    public void changeUserPassword(String password) throws Exception
    {
        user.setPassword(password);
        lookupEJB(UserService.class).merge(user);
        passwordChanged = true;
    }

    private void createLoginHistoryRecord()
    {
        try
        {
            lookupEJB(LoginHistoryService.class).create(new LoginHistory(new Date().toString()));
        }
        catch(Exception ex)
        {
            getFacesContext().addMessage(null, new FacesMessage(ex.getMessage()));
            ex.printStackTrace();
        }
    }

    @SuppressWarnings("unchecked")
    public static <X> X lookupEJB(Class<X> ejbClass)
    {
        X result = null;
        try
        {
            InitialContext ctx = new InitialContext();
            String uri = "java:global/" + ctx.lookup("java:app/AppName") + "/" + ejbClass.getSimpleName();
            result = (X)(ctx.lookup(uri));
            ctx.close();
        }
        catch (ClassCastException | NamingException ex)
        {
            ex.printStackTrace();
            throw new IllegalStateException(ex);
        }
        return result;
    }

    public FacesContext getFacesContext() { return FacesContext.getCurrentInstance(); }
    public ExternalContext getExternalContext() { return getFacesContext().getExternalContext(); }
    public HttpServletRequest getRequest() { return (HttpServletRequest)getExternalContext().getRequest(); }

    public boolean isPasswordChanged() { return passwordChanged; }
    public void setPasswordChanged(boolean passwordChanged) { this.passwordChanged = passwordChanged; }
}